//
//  ViewController.swift
//  videochat_ios_demo
//
//  Created by George Mo on 2024/8/16.
//

import UIKit
import videochat_ios_sdk

class ConfigViewController: UIViewController {
    
    private var selectedMode: TYVoiceChatMode = .tap2talk
    
    lazy var modeSegmentControl: UISegmentedControl = {
        let sc = UISegmentedControl(frame: .zero)
        sc.insertSegment(action: UIAction(title: "tap2talk", handler: { _ in
            self.selectedMode = .tap2talk
        }), at: 0, animated: false)
        sc.insertSegment(action: UIAction(title: "双工duplex", handler: { _ in
            self.selectedMode = .duplex
        }), at: 1, animated: false)
        sc.insertSegment(action: UIAction(title: "push2talk", handler: { _ in
            self.selectedMode = .push2talk
        }), at: 2, animated: false)
        sc.selectedSegmentIndex = 0
        return sc
    }()
    
    /**
     数字人资产id
     数字人模型id
     音色id
     角色id
     API key
     **/
    
    lazy var avatarAssetLabel: UILabel = {
        let l = UILabel()
        l.text = "数字人ID"
        return l
    }()
    lazy var avatarAssetTextInput: UITextField = {
        let l = UITextField()
        l.placeholder = ""
        l.borderStyle = .line
        l.returnKeyType = .done
        l.delegate = self
        return l
    }()
    
    lazy var avatarModelLabel: UILabel = {
        let l = UILabel()
        l.text = "AvatarModelID"
        return l
    }()
    lazy var avatarModelTextInput: UITextField = {
        let l = UITextField()
        l.placeholder = ""
        l.borderStyle = .line
        l.returnKeyType = .done
        l.delegate = self
        return l
    }()
    
    lazy var voiceLabel: UILabel = {
        let l = UILabel()
        l.text = "音色ID"
        return l
    }()
    lazy var voiceTextInput: UITextField = {
        let l = UITextField()
        l.placeholder = ""
        l.borderStyle = .line
        l.returnKeyType = .done
        l.delegate = self
        return l
    }()
    
    lazy var characterLabel: UILabel = {
        let l = UILabel()
        l.text = "应用ID"
        
        return l
    }()
    lazy var characterTextInput: UITextField = {
        let l = UITextField()
        l.placeholder = ""
        l.borderStyle = .line
        l.returnKeyType = .done
        l.delegate = self
        return l
    }()
    
    lazy var apiKeyLabel: UILabel = {
        let l = UILabel()
        l.text = "API Key"
        return l
    }()
    lazy var apiKeyTextInput: UITextField = {
        let l = UITextField()
        l.borderStyle = .line
        l.placeholder = ""
        l.returnKeyType = .done
        l.delegate = self
        return l
    }()
    
    lazy var llmLabel: UILabel = {
        let l = UILabel()
        l.text = "LLM模型ID"
        return l
    }()
    lazy var llmTextInput: UITextField = {
        let l = UITextField()
        l.borderStyle = .line
        l.placeholder = ""
        l.returnKeyType = .done
        l.delegate = self
        l.text = "xingchen-plus-v2"
        return l
    }()
    
    lazy var startButton: UIButton = {
        let b = UIButton()
        b.setTitle("生成数字人", for: [])
        b.backgroundColor = UIColor.aly_fromHex(0x49419E)
        b.addTarget(self, action: #selector(startButtonTapped), for: .touchUpInside)
        return b
    }()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        view.backgroundColor = .white
        
        view.addSubview(modeSegmentControl)
        
        let stackView = UIStackView()
        stackView.axis = .vertical
        stackView.alignment = .fill
        stackView.spacing = 4
        stackView.addArrangedSubview(llmLabel)
        stackView.addArrangedSubview(llmTextInput)
        stackView.setCustomSpacing(12, after: llmTextInput)
        stackView.addArrangedSubview(characterLabel)
        stackView.addArrangedSubview(characterTextInput)
        stackView.setCustomSpacing(12, after: characterTextInput)
        stackView.addArrangedSubview(avatarAssetLabel)
        stackView.addArrangedSubview(avatarAssetTextInput)
        stackView.setCustomSpacing(12, after: avatarAssetTextInput)
        stackView.addArrangedSubview(avatarModelLabel)
        stackView.addArrangedSubview(avatarModelTextInput)
        stackView.setCustomSpacing(12, after: avatarModelTextInput)
        stackView.addArrangedSubview(voiceLabel)
        stackView.addArrangedSubview(voiceTextInput)
        stackView.setCustomSpacing(12, after: voiceTextInput)
        stackView.addArrangedSubview(apiKeyLabel)
        stackView.addArrangedSubview(apiKeyTextInput)
        
        view.addSubview(stackView)
        view.addSubview(startButton)
        
        modeSegmentControl.snp.makeConstraints { make in
            make.top.equalToSuperview().inset(60)
            make.centerX.equalToSuperview()
        }
        stackView.snp.makeConstraints { make in
            make.top.equalTo(modeSegmentControl.snp.bottom).offset(40)
            make.leading.trailing.equalToSuperview().inset(12)
        }
        startButton.snp.makeConstraints { make in
            make.leading.trailing.equalToSuperview().inset(12)
            make.bottom.equalToSuperview().inset(40)
            make.height.equalTo(40)
        }
    }
    
    @objc func startButtonTapped() {
        let voice = voiceTextInput.text ?? ""
        let avatarId = avatarAssetTextInput.text ?? ""
        let apiKey = apiKeyTextInput.text ?? ""
        let charaterId = characterTextInput.text ?? ""
        let avatarModelId = avatarModelTextInput.text ?? ""
        let llmId = llmTextInput.text ?? ""
        
        // usePlatformChatHistory为true时，同一个CharacterId+UserId的组合会记住历史对话上下文，可在此配置
        // 实际使用时，最好给每个用户分配一个单独的userId，此处使用时间戳
        let usePlatformChatHistory = false
        let userId = "testUserId_\(Date().timeIntervalSince1970)"

        if !voice.isEmpty,
           !avatarId.isEmpty,
           !apiKey.isEmpty,
           !avatarModelId.isEmpty,
           !llmId.isEmpty{
            
            let vc = VideoChatViewController(type: .Avatar,
                                             modelId: llmId,
                                             voice: voice,
                                             avatarId: avatarId,
                                             avatarModelId: avatarModelId,
                                             characterId: charaterId.isEmpty ? nil : charaterId,
                                             voicePlatform: nil,
                                             customVoiceModel: nil,
                                             mode: self.selectedMode,
                                             endpointUrl: "https://nlp.aliyuncs.com/v2/api/videochat/initialize",
                                             apiKey: apiKey,
                                             keepAlive: false,
                                             usePlatformChatHistory: usePlatformChatHistory,
                                             userId: userId)
            navigationController?.pushViewController(vc, animated: true)
        } else {
            ALYToast.shared.showToast(text: "参数配置不完整，请检查", style: .error)
        }
    }
}

extension ConfigViewController: UITextFieldDelegate {
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        textField.resignFirstResponder()
        return true
    }
}
